@@ -30,6 +30,18 @@ predict.manyglm=function(object,newdata = NULL,type = c("link", "response",
  	
 	fm <- formula(object)
 
+	if( "K" %in% names(object$call) ) #for predictions when K is non-zero:
+	{
+	  K = eval(object$call$K)
+	  if(K>1 & object$family!="binomial")
+	  {
+	    warning("Argument K should only be specified when family is binomial, K reset to 1")
+	    K=1
+	  }      
+	}
+	else
+	  K = 1
+    
 	# use predict.glm to compute each column one at a time
 	for(iVar in 1:nVar)
 	{
@@ -39,23 +51,33 @@ predict.manyglm=function(object,newdata = NULL,type = c("link", "response",
 	             "gaussian"=gaussian(),
 	             "negative.binomial"=negative.binomial(theta=object$theta[iVar])
         	)
-                form <- as.formula(paste("object$y[ ,", iVar, "] ~ ", fm[3]))
 	        if(is.null(object$data))
 	            dat.i = model.frame(object)
 	        else
-                    dat.i = data.frame(object$y[,iVar], object$data)
-                object.i = glm(form, family=fam, data=dat.i)
-
-                ft.i <- predict.glm(object.i, newdata=newdata, se.fit=se.fit, 
+              dat.i = data.frame(object$y[,iVar], object$data)
+          if(K>1)
+          {
+            dat.i$K=K
+            form <- as.formula(paste("object$y[ ,", iVar, "]/K ~ ", fm[3]))
+            object.i = glm(form, family=fam, data=dat.i, weights=K, start=as.vector(object$coef[,iVar])) 
+          }
+          else
+          {
+            form <- as.formula(paste("object$y[ ,", iVar, "] ~ ", fm[3]))
+            object.i = glm(form, family=fam, data=dat.i, start=as.vector(object$coef[,iVar]))
+            #DW, 18/11/14:  starting value included in predict to avoid errors with sparse data
+          }
+	       
+          ft.i <- predict.glm(object.i, newdata=newdata, se.fit=se.fit, 
                                  type=type, terms = terms, na.action = na.action)
-                if(se.fit==T)
-		{
+          if(se.fit==T)
+		      {
 	            fts[,iVar] = ft.i$fit
          	    ses[,iVar] = ft.i$se
-		}
-		else
-		    fts[,iVar] = ft.i
-	}
+      		}
+		      else
+		        fts[,iVar] = ft.i
+  }
 	if(se.fit)
 		out = list(fit=fts,se.fit=ses)
 	else
